@@ -43,8 +43,16 @@ def get_auth_conf(kwargs):
         # For DBAuth, hoist the user.column into as the id: for the URL
         user_column = auth_kwargs.get('user', {}).get('column', 'user')
         data_conf = gramex.handlers.DBAuth.clear_special_keys(
-            auth_kwargs.copy(), 'rules', 'user', 'password', 'forgot',
-            'signup', 'template', 'delay', 'email_column')
+            auth_kwargs.copy(),
+            'rules',
+            'user',
+            'password',
+            'forgot',
+            'signup',
+            'template',
+            'delay',
+            'email_column',
+        )
         data_conf['id'] = user_column
         return authhandler, auth_conf, data_conf
     else:
@@ -80,9 +88,7 @@ class AdminFormHandler(gramex.handlers.FormHandler):
         subject = self.signup['email_subject']
         body = self.signup['email_text']
         yield gramex.service.threadpool.submit(
-            mailer.mail,
-            to=to, subject=subject.format(**user),
-            body=body.format(**user)
+            mailer.mail, to=to, subject=subject.format(**user), body=body.format(**user)
         )
 
     @classmethod
@@ -95,7 +101,8 @@ class AdminFormHandler(gramex.handlers.FormHandler):
                 raise ValueError(f'admin_kwargs not found in {cls.name}.')
             cls.signup.update(admin_kwargs.pop('signup', {}))
             cls.authhandler, cls.auth_conf, data_conf = get_auth_conf(
-                kwargs.get('admin_kwargs', {}))
+                kwargs.get('admin_kwargs', {})
+            )
             # When this class is set up for rules, and the authhandler has rules...
             if kwargs.get('rules', False) and cls.auth_conf.kwargs.get('rules', False):
                 # Get the rules for formhandler
@@ -106,9 +113,9 @@ class AdminFormHandler(gramex.handlers.FormHandler):
                 for url in gramex.conf['url']:
                     if url == authhandler or url.endswith(f':{authhandler}'):
                         break
-                data_conf = gramex.conf['url'].get(
-                    url, {}
-                ).get('kwargs', {}).get('rules', {}).copy()
+                data_conf = (
+                    gramex.conf['url'].get(url, {}).get('kwargs', {}).get('rules', {}).copy()
+                )
                 data_conf['id'] = ['selector', 'pattern']
         except ValueError as e:
             super(gramex.handlers.FormHandler, cls).setup(**kwargs)
@@ -149,10 +156,11 @@ def evaluate(handler, code):
     try:
         context = contexts.setdefault(handler.session['id'], {})
         context['handler'] = handler
+        # B307:eval B102:exec_used is safe since only admin can run this
         if mode == 'eval':
-            result = eval(co, context)  # nosec: only admin can run this
+            result = eval(co, context)  # nosec B307
         else:
-            exec(co, context)           # nosec: only admin can run this
+            exec(co, context)  # nosec B102
             result = None
     except Exception as e:
         result = e
@@ -172,6 +180,7 @@ def system_information(handler):
     value, error = {}, {}
     try:
         import psutil
+
         process = psutil.Process(os.getpid())
         value['system', 'cpu-count'] = psutil.cpu_count()
         value['system', 'cpu-usage'] = psutil.cpu_percent()
@@ -189,23 +198,26 @@ def system_information(handler):
         error['gramex', 'open-files'] = 'psutil not installed'
     try:
         import conda
-        value['conda', 'version'] = conda.__version__,
+
+        value['conda', 'version'] = (conda.__version__,)
     except ImportError:
         app_log.warning('conda required for conda stats')
         error['conda', 'version'] = 'conda not installed'
 
-    from shutilwhich import which
+    from shutil import which
+
     value['node', 'path'] = which('node')
     value['git', 'path'] = which('git')
 
     from gramex.cache import Subprocess
+
     apps = {
-        # shell=True is safe here since the code is constructed entirely in this function
-        # We use shell to pick up the commands' paths from the shell.
-        ('node', 'version'): Subprocess('node --version', shell=True),  # nosec
-        ('npm', 'version'): Subprocess('npm --version', shell=True),    # nosec
-        ('yarn', 'version'): Subprocess('yarn --version', shell=True),  # nosec
-        ('git', 'version'): Subprocess('git --version', shell=True),    # nosec
+        # B602:any_other_function_with_shell_equals_true is safe here since the code is
+        # constructed entirely in this function. We use shell to pick up the commands' paths.
+        ('node', 'version'): Subprocess('node --version', shell=True),  # nosec 602
+        ('npm', 'version'): Subprocess('npm --version', shell=True),  # nosec 602
+        ('yarn', 'version'): Subprocess('yarn --version', shell=True),  # nosec 602
+        ('git', 'version'): Subprocess('git --version', shell=True),  # nosec 602
     }
     for key, proc in apps.items():
         stdout, stderr = yield proc.wait_for_exit()
@@ -219,6 +231,7 @@ def system_information(handler):
     value['gramex', 'path'] = os.path.dirname(gramex.__file__)
 
     import pandas as pd
+
     df = pd.DataFrame({'value': value, 'error': error}).reset_index()
     df.columns = ['section', 'key'] + df.columns[2:].tolist()
     df = df[['section', 'key', 'value', 'error']].sort_values(['section', 'key'])
@@ -282,10 +295,12 @@ def schedule(handler, service):
                 raise arg['error']
         for result in results:
             if 'html' in result:
+
                 def _img(match):
                     path = result['images'][match.group(1)]
                     img = gramex.cache.open(path, 'bin', transform=b2a_base64)
                     url = b'data:image/png;base64,' + img.replace(b'\n', b'')
                     return url.decode('utf-8')
+
                 result['html'] = re.sub(r'cid:([^\'"\s]+)', _img, result['html'])
         raise Return(json.dumps(results))
