@@ -43,12 +43,13 @@ from yaml.constructor import ConstructorError
 from orderedattrdict import AttrDict, DefaultAttrDict
 from slugify import slugify
 from errno import EACCES, EPERM
+
 # We don't use six -- but import into globals() for _yaml_open().
 # This allows YAML conditionals like `key if six.text_type(...): val`
-import six      # noqa
+import six  # noqa
 
 
-ERROR_SHARING_VIOLATION = 32        # from winerror.ERROR_SHARING_VIOLATION
+ERROR_SHARING_VIOLATION = 32  # from winerror.ERROR_SHARING_VIOLATION
 
 # gramex.config.app_log is the default logger used by all of gramex
 # If it's not there, create one.
@@ -95,14 +96,13 @@ def walk(node):
         RuntimeError: maximum recursion depth exceeded
     '''
     if hasattr(node, 'items'):
+        # Convert note.items() to list to prevent keys changing during iteration
         for key, value in list(node.items()):
-            for item in walk(value):
-                yield item
+            yield from walk(value)
             yield key, value, node
     elif isinstance(node, list):
         for index, value in enumerate(node):
-            for item in walk(value):
-                yield item
+            yield from walk(value)
             yield index, value, node
 
 
@@ -153,7 +153,7 @@ class ChainConfig(AttrDict):
     def __pos__(self):
         '''+config returns layers merged in order, removing null keys'''
         conf = AttrDict()
-        for name, config in self.items():
+        for _name, config in self.items():
             if hasattr(config, '__pos__'):
                 config.__pos__()
             merge(old=conf, new=config, mode='overwrite')
@@ -195,7 +195,8 @@ def setup_variables():
             variables['GRAMEXDATA'] = os.path.join(variables['LOCALAPPDATA'], 'Gramex Data')
         elif sys.platform == 'darwin':
             variables['GRAMEXDATA'] = os.path.expanduser(
-                '~/Library/Application Support/Gramex Data')
+                '~/Library/Application Support/Gramex Data'
+            )
         else:
             variables['GRAMEXDATA'] = os.path.abspath('.')
             app_log.warning(f'$GRAMEXDATA set to {variables["GRAMEXDATA"]} for OS {sys.platform}')
@@ -227,23 +228,31 @@ def _substitute_variable(val):
 
 
 def _calc_value(val, key):
+    '''Calculate the value to assign to this key.
+
+    If val is a scalar (string, boolean, dict, etc), return as-is.
+
+    If it's a list or a dict, return calculated values of underlying values.
+
+    If it's a dict with a `function` key, evaluation the function and return the non-None value.
+    If the returned value(s) are None, return the calculated 'default' value.
     '''
-    Calculate the value to assign to this key.
+    if isinstance(val, dict):
+        if val.get('function'):
+            from .transforms import build_transform
 
-    If ``val`` is not a dictionary that has a ``function`` key, return it as-is.
-
-    If it has a function key, call that function (with specified args, kwargs,
-    etc) and allow the ``key`` parameter as an argument.
-
-    If the function is a generator, the first value is used.
-    '''
-    if hasattr(val, 'get') and val.get('function'):
-        from .transforms import build_transform
-        function = build_transform(val, vars={'key': None}, filename=f'config:{key}')
-        for result in function(key):
-            if result is not None:
-                return result
-        return val.get('default')
+            function = build_transform(val, vars={'key': None}, filename=f'config:{key}')
+            try:
+                for result in function(key):
+                    if result is not None:
+                        return result
+            except Exception:
+                app_log.exception(f'Error in calculated variable: {key}: {val}')
+            return _calc_value(val.get('default', None), key)
+        else:
+            return {k: _calc_value(v, k) for k, v in val.items()}
+    elif isinstance(val, (list, tuple)):
+        return [_calc_value(v, key) for v in val]
     else:
         return _substitute_variable(val)
 
@@ -253,7 +262,8 @@ _valid_key_chars = string.ascii_letters + string.digits
 
 def random_string(size, chars=_valid_key_chars):
     '''Return random string of length size using chars (which defaults to alphanumeric)'''
-    return ''.join(choice(chars) for index in range(size))   # nosec: non-cryptographic use
+    # B311:random random() is safe since it's for non-cryptographic use
+    return ''.join(choice(chars) for index in range(size))  # nosec B311
 
 
 RANDOM_KEY = r'$*'
@@ -268,7 +278,8 @@ def _from_yaml(loader, node):
     yield attrdict
     if not isinstance(node, MappingNode):
         raise ConstructorError(
-            None, None, f'expected a mapping node, but found {node.id}', node.start_mark)
+            None, None, f'expected a mapping node, but found {node.id}', node.start_mark
+        )
     loader.flatten_mapping(node)
     for key_node, value_node in node.value:
         key = loader.construct_object(key_node, deep=False)
@@ -280,12 +291,18 @@ def _from_yaml(loader, node):
             hash(key)
         except TypeError as exc:
             raise ConstructorError(
-                'while constructing a mapping', node.start_mark,
-                f'found unacceptable key ({exc})', key_node.start_mark)
+                'while constructing a mapping',
+                node.start_mark,
+                f'found unacceptable key ({exc})',
+                key_node.start_mark,
+            )
         if key in attrdict:
             raise ConstructorError(
-                'while constructing a mapping', node.start_mark,
-                f'found duplicate key ({key})', key_node.start_mark)
+                'while constructing a mapping',
+                node.start_mark,
+                f'found duplicate key ({key})',
+                key_node.start_mark,
+            )
         attrdict[key] = loader.construct_object(value_node, deep=False)
 
 
@@ -297,6 +314,7 @@ class ConfigYAMLLoader(SafeLoader):
 
     If there are duplicate keys, this raises an error.
     '''
+
     def __init__(self, *args, **kwargs):
         super(ConfigYAMLLoader, self).__init__(*args, **kwargs)
         self.add_constructor('tag:yaml.org,2002:map', _from_yaml)
@@ -325,7 +343,8 @@ def _yaml_open(path, default=AttrDict(), **kwargs):
     app_log.debug(f'Loading config: {path}')
     with path.open(encoding='utf-8') as handle:
         try:
-            result = yaml.load(handle, Loader=ConfigYAMLLoader)     # nosec: SafeLoader
+            # B506:yaml_load we use a safe loader
+            result = yaml.load(handle, Loader=ConfigYAMLLoader)  # nosec B506
         except Exception:
             app_log.exception(f'Config error: {path}')
             return default
@@ -336,8 +355,8 @@ def _yaml_open(path, default=AttrDict(), **kwargs):
 
     # Variables based on YAML file location
     yaml_path = str(path.parent)
-    kwargs.setdefault('YAMLPATH', yaml_path)    # Path to YAML folder
-    kwargs.setdefault('YAMLFILE', str(path))    # Path to YAML file
+    kwargs.setdefault('YAMLPATH', yaml_path)  # Path to YAML folder
+    kwargs.setdefault('YAMLFILE', str(path))  # Path to YAML file
     # $YAMLURL defaults to the relative URL from cwd to YAML folder.
     try:
         yamlurl = os.path.relpath(yaml_path)
@@ -368,13 +387,13 @@ def _yaml_open(path, default=AttrDict(), **kwargs):
     # Evaluate conditionals. "x if cond: y" becomes "x: y" if cond evals to True
     remove, replace = [], []
     frozen_vars = dict(variables)
-    for key, value, node in walk(result):
+    for key, _value, node in walk(result):
         if isinstance(key, str) and ' if ' in key:
             # Evaluate conditional
             base, expr = key.split(' if ', 2)
             try:
-                # eval() is safe here since `expr` is written by app developer
-                condition = eval(expr, globals(), frozen_vars)  # nosec: developer-initiated
+                # B307:eval this is safe since `expr` is written by app developer
+                condition = eval(expr, globals(), frozen_vars)  # nosec B307
             except Exception:
                 condition = False
                 app_log.exception(f'Failed condition evaluation: {key}')
@@ -586,6 +605,7 @@ class PathConfig(AttrDict):
             The ``os.stat()`` information about this file (or ``None`` if the
             file is missing.)
     '''
+
     duplicate_warn = None
 
     def __init__(self, path, warn=None):
@@ -604,12 +624,15 @@ class PathConfig(AttrDict):
         # ... or if an imported file is deleted / updated
         for imp in self.__info__.imports:
             exists = imp.path.exists()
+            # If the path existed but has now been deleted, log it
             if not exists and imp.stat is not None:
                 reload = True
-                app_log.info(f'No config found: {imp.path}')
+                app_log.debug(f'Config deleted: {imp.path}')
                 break
-            if exists and (imp.path.stat().st_mtime > imp.stat.st_mtime or
-                           imp.path.stat().st_size != imp.stat.st_size):
+            if exists and (
+                imp.path.stat().st_mtime > imp.stat.st_mtime
+                or imp.path.stat().st_size != imp.stat.st_size
+            ):
                 reload = True
                 app_log.info(f'Updated config: {imp.path}')
                 break
@@ -649,13 +672,15 @@ class CustomJSONEncoder(JSONEncoder):
     '''
     Encodes object to JSON, additionally converting datetime into ISO 8601 format
     '''
+
     def default(self, obj):
         import numpy as np
 
         if hasattr(obj, 'to_dict'):
             # Slow but reliable. Handles conversion of numpy objects, mixed types, etc.
-            return loads(obj.to_json(orient='records', date_format='iso'),
-                         object_pairs_hook=OrderedDict)
+            return loads(
+                obj.to_json(orient='records', date_format='iso'), object_pairs_hook=OrderedDict
+            )
         elif isinstance(obj, datetime.datetime):
             # Use local timezone if no timezone is specified
             if obj.tzinfo is None:
@@ -663,7 +688,7 @@ class CustomJSONEncoder(JSONEncoder):
             return obj.isoformat()
         elif isinstance(obj, np.datetime64):
             obj = obj.item()
-            if (isinstance(obj, datetime.datetime) and obj.tzinfo is None):
+            if isinstance(obj, datetime.datetime) and obj.tzinfo is None:
                 obj = obj.replace(tzinfo=dateutil.tz.tzlocal())
             return obj.isoformat()
         elif isinstance(obj, np.integer):
@@ -683,6 +708,7 @@ class CustomJSONDecoder(JSONDecoder):
     '''
     Decodes JSON string, converting ISO 8601 datetime to datetime
     '''
+
     # Check if a string might be a datetime. Handles variants like:
     # 2001-02-03T04:05:06Z
     # 2001-02-03T04:05:06+000
@@ -758,6 +784,7 @@ class TimedRotatingCSVHandler(logging.handlers.TimedRotatingFileHandler):
     column keys. When ``.emit()`` is called, it expects an object with the
     same keys as ``keys``.
     '''
+
     def __init__(self, *args, **kwargs):
         self.keys = kwargs.pop('keys')
         super(TimedRotatingCSVHandler, self).__init__(*args, **kwargs)
@@ -796,6 +823,7 @@ class TimedRotatingCSVHandler(logging.handlers.TimedRotatingFileHandler):
 
 def ioloop_running(loop):
     '''Returns whether the Tornado ioloop is running on not'''
+    # TODO: Pressing Ctrl+C may cause this to raise an exception. Explore how to handle that
     return loop.asyncio_loop.is_running()
 
 
@@ -809,6 +837,11 @@ def used_kwargs(method, kwargs, ignore_keywords=False):
     If the method uses ``**kwargs`` (keywords), it uses all keys. To ignore this
     and return only named arguments, use ``ignore_keywords=True``.
     '''
+    # In Pandas 1.5, DataFrame.to_csv and DataFrame.to_excel are wrapped with @deprecate_kwargs.
+    # We dive deeper to detect the actual keywords. __wrapped__ is provided by functools.wraps
+    # https://docs.python.org/3/library/functools.html
+    while hasattr(method, '__wrapped__'):
+        method = method.__wrapped__
     argspec = inspect.getfullargspec(method)
     # If method uses **kwargs, return all kwargs (unless you ignore **kwargs)
     if argspec.varkw and not ignore_keywords:
@@ -853,9 +886,11 @@ def setup_secrets(path, max_age_days=1000000, clear=True):
     if secrets_url and secrets_key:
         from urllib.request import urlopen
         from tornado.web import decode_signed_value
+
         app_log.info(f'Fetching remote secrets from {secrets_url}')
         # Load string from the URL -- but ignore comments. file:// URLs are fine too
-        value = yaml.safe_load(urlopen(secrets_url))    # nosec: allow file:// URLs
+        # B310:urllib_urlopen secrets can be local files or URLs
+        value = yaml.safe_load(urlopen(secrets_url))  # nosec B310
         value = decode_signed_value(secrets_key, '', value, max_age_days=max_age_days)
         result.update(loads(value.decode('utf-8')))
     # If SECRETS_IMPORT: is set, fetch secrets from those file(s) as well.
@@ -863,9 +898,13 @@ def setup_secrets(path, max_age_days=1000000, clear=True):
     secrets_import = result.pop('SECRETS_IMPORT', None)
     if secrets_import:
         # Create a list of file patterns to import from
-        imports = (list(secrets_import.values()) if isinstance(secrets_import, dict) else
-                   secrets_import if isinstance(secrets_import, (list, tuple)) else
-                   [secrets_import])
+        imports = (
+            list(secrets_import.values())
+            if isinstance(secrets_import, dict)
+            else secrets_import
+            if isinstance(secrets_import, (list, tuple))
+            else [secrets_import]
+        )
         for pattern in imports:
             for import_path in path.parent.glob(pattern):
                 setup_secrets(import_path, max_age_days, clear=False)
