@@ -1,10 +1,6 @@
 from fnmatch import fnmatch
-import io
 import os
-import csv
 import json
-import time
-import uuid
 import logging
 import tornado.escape
 import tornado.httpclient
@@ -17,24 +13,33 @@ from orderedattrdict import AttrDict
 import gramex
 import gramex.cache
 from gramex.http import UNAUTHORIZED, FORBIDDEN
-from gramex.config import app_log, objectpath, merge
+from gramex.config import app_log, objectpath, merge, CustomJSONEncoder, CustomJSONDecoder
 from gramex.transforms import build_transform
 from .basehandler import BaseHandler, build_log_info
 
 _folder = os.path.dirname(os.path.abspath(__file__))
 _auth_template = os.path.join(_folder, 'auth.template.html')
-_user_info_path = os.path.join(gramex.variables.GRAMEXDATA, 'auth.user.db')
-_user_info = gramex.cache.SQLiteStore(_user_info_path, table='user')
 
 
 class AuthHandler(BaseHandler):
     '''The parent handler for all Auth handlers.'''
+
     _RECAPTCHA_VERIFY_URL = 'https://www.google.com/recaptcha/api/siteverify'
 
     @classmethod
-    def setup(cls, prepare=None, action=None, delay=None, session_expiry=None,
-              session_inactive=None, user_key='user', lookup=None, recaptcha=None,
-              rules=None, **kwargs):
+    def setup(
+        cls,
+        prepare=None,
+        action=None,
+        delay=None,
+        session_expiry=None,
+        session_inactive=None,
+        user_key='user',
+        lookup=None,
+        recaptcha=None,
+        rules=None,
+        **kwargs,
+    ):
         # Set up default redirection based on ?next=...
         if 'redirect' not in kwargs:
             kwargs['redirect'] = AttrDict([('query', 'next'), ('header', 'Referer')])
@@ -42,10 +47,10 @@ class AuthHandler(BaseHandler):
         super(AuthHandler, cls).setup(**kwargs)
 
         # Set up logging for login/logout events
-        logger = logging.getLogger('gramex.user')
         keys = objectpath(gramex.conf, 'log.handlers.user.keys', [])
-        log_info = build_log_info(keys, 'event')
-        cls.log_user_event = lambda handler, event: logger.info(log_info(handler, event))
+        cls.log_info = build_log_info(keys, 'event')
+        keys = list(objectpath(gramex.conf, 'storelocations.userlog.columns', {}).keys())
+        cls.user_log_info = build_log_info(keys, 'event')
 
         # Count failed logins
         cls.failed_logins = Counter()
@@ -83,7 +88,8 @@ class AuthHandler(BaseHandler):
                 conf={'function': prepare},
                 vars={'handler': None, 'args': None},
                 filename=f'url:{cls.name}:prepare',
-                iter=False)
+                iter=False,
+            )
         # Prepare recaptcha
         if recaptcha is not None:
             if 'key' not in recaptcha:
@@ -100,16 +106,25 @@ class AuthHandler(BaseHandler):
             if not isinstance(action, list):
                 action = [action]
             for conf in action:
-                cls.actions.append(build_transform(
-                    conf, vars=AttrDict(handler=None),
-                    filename=f'url:{cls.name}:{conf.function}'))
+                cls.actions.append(
+                    build_transform(
+                        conf, vars={'handler': None}, filename=f'url:{cls.name}:{conf.function}'
+                    )
+                )
+
+    def log_user_event(self, event, user=None):
+        info = self.log_info(event)
+        logging.getLogger('gramex.user').info(info)
+        args = {key: [val] for key, val in self.user_log_info(event).items()}
+        gramex.data.insert(**gramex.service.storelocations.userlog, args=args)
 
     @coroutine
     def prepare(self):
         super(AuthHandler, self).prepare()
         if 'prepare' in self.auth_methods:
             result = yield gramex.service.threadpool.submit(
-                self.auth_methods['prepare'], handler=self, args=self.args)
+                self.auth_methods['prepare'], handler=self, args=self.args
+            )
             if result is not None:
                 self.args = result
         if 'recaptcha' in self.auth_methods:
@@ -118,9 +133,16 @@ class AuthHandler(BaseHandler):
     @staticmethod
     def update_user(_user_id, **kwargs):
         '''Update user login/logout event.'''
-        info = _user_info.load(_user_id)
+        args = {'key': [_user_id]}
+        results = gramex.data.filter(**gramex.service.storelocations.user, args=args)
+        info = json.loads(results.value.iloc[0], cls=CustomJSONDecoder) if len(results) else {}
         info.update(kwargs)
-        _user_info.dump(_user_id, info)
+        args['value'] = [json.dumps(info, ensure_ascii=True, cls=CustomJSONEncoder)]
+        if len(results) == 0:
+            gramex.data.insert(**gramex.service.storelocations.user, args=args)
+        else:
+            gramex.data.update(**gramex.service.storelocations.user, id='key', args=args)
+            # TODO: If there are more than 1 results, we may want to delete the extras
         return info
 
     @coroutine
@@ -152,13 +174,17 @@ class AuthHandler(BaseHandler):
         if self.lookup is not None:
             # Look up the user ID in the lookup table and fetch all matching rows
             users = yield gramex.service.threadpool.submit(
-                gramex.data.filter, args={self.lookup_id: [user['id']]}, **self.lookup)
+                gramex.data.filter, args={self.lookup_id: [user['id']]}, **self.lookup
+            )
             if len(users) > 0 and self.lookup_id in users.columns:
                 # Update the user attributes with the non-null items in the looked up row
-                user.update({
-                    key: val for key, val in users.iloc[0].iteritems()
-                    if not gramex.data.pd.isnull(val)
-                })
+                user.update(
+                    {
+                        key: val
+                        for key, val in users.iloc[0].items()
+                        if not gramex.data.pd.isnull(val)
+                    }
+                )
 
         # Persist user attributes (e.g. refresh_token from Google auth.)
         # If new user object doesn't have anything from previous login, restore it.
@@ -207,11 +233,9 @@ class AuthHandler(BaseHandler):
         token = self.get_argument('recaptcha', None)
         if token is None:
             raise HTTPError(FORBIDDEN, "'recaptcha' argument missing from POST")
-        body = urlencode({
-            'secret': conf.secret,
-            'response': token,
-            'remoteip': self.request.remote_ip
-        })
+        body = urlencode(
+            {'secret': conf.secret, 'response': token, 'remoteip': self.request.remote_ip}
+        )
         http = tornado.httpclient.AsyncHTTPClient()
         response = yield http.fetch(self._RECAPTCHA_VERIFY_URL, method='POST', body=body)
         result = json.loads(response.body)
@@ -242,16 +266,15 @@ class GoogleAuth(AuthHandler, GoogleOAuth2Mixin):
     def get(self):
         self.settings[self._OAUTH_SETTINGS_KEY] = {
             'key': self.kwargs['key'],
-            'secret': self.kwargs['secret']
+            'secret': self.kwargs['secret'],
         }
         code = self.get_arg('code', '')
         if code:
-            access = yield self.get_authenticated_user(
-                redirect_uri=self.xredirect_uri,
-                code=code)
+            access = yield self.get_authenticated_user(redirect_uri=self.xredirect_uri, code=code)
             user = yield self.oauth2_request(
                 'https://www.googleapis.com/oauth2/v1/userinfo',
-                access_token=access['access_token'])
+                access_token=access['access_token'],
+            )
             merge(user, access, mode='setdefault')
             yield self.set_user(user, id='email')
             self.session['google_access_token'] = access['access_token']
@@ -268,7 +291,8 @@ class GoogleAuth(AuthHandler, GoogleOAuth2Mixin):
                 client_id=self.kwargs['key'],
                 scope=scope,
                 response_type='code',
-                extra_params=self.kwargs.get('extra_params', {}))
+                extra_params=self.kwargs.get('extra_params', {}),
+            )
 
     @classmethod
     @coroutine
@@ -301,12 +325,14 @@ class GoogleAuth(AuthHandler, GoogleOAuth2Mixin):
                 refresh_token = user['refresh_token']
             else:
                 raise HTTPError(FORBIDDEN, "No refresh_token provided")
-        body = urlencode({
-            'grant_type': 'refresh_token',
-            'client_id': cls.kwargs['key'],
-            'client_secret': cls.kwargs['secret'],
-            'refresh_token': refresh_token,
-        })
+        body = urlencode(
+            {
+                'grant_type': 'refresh_token',
+                'client_id': cls.kwargs['key'],
+                'client_secret': cls.kwargs['secret'],
+                'refresh_token': refresh_token,
+            }
+        )
         http = tornado.httpclient.AsyncHTTPClient()
         response = yield http.fetch(cls._OAUTH_ACCESS_TOKEN_URL, method='POST', body=body)
         result = json.loads(response.body)
@@ -360,6 +386,7 @@ class SimpleAuth(AuthHandler):
     3. Application checks username and password. On match, redirects.
     4. On any error, shows template (with error)
     '''
+
     @classmethod
     def setup(cls, **kwargs):
         super(SimpleAuth, cls).setup(**kwargs)
@@ -392,55 +419,3 @@ class SimpleAuth(AuthHandler):
             self.log_user_event(event='fail')
             self.set_status(UNAUTHORIZED)
             self.render_template(self.template, error={'code': 'auth', 'error': 'Cannot log in'})
-
-
-class OTP(object):
-    '''
-    OTP: One-time password. Also used for password recovery
-    '''
-    def __init__(self, size=None):
-        '''
-        Set up the database that stores password recovery tokens.
-        ``size`` is the length of the OTP in characters. Defaults to the
-        full hashing string
-        '''
-        self.size = size
-        # Alter the table to ensure it has user, email, token, expire columns
-        gramex.data.alter(**gramex.service.otp, columns={
-            'user': 'TEXT',
-            'email': 'TEXT',
-            'token': 'TEXT',
-            'expire': 'REAL',
-        })
-
-    def token(self, user, email, expire):
-        '''Generate a one-tie token, store it in the recovery database, and return it'''
-        token = uuid.uuid4().hex[:self.size]
-        gramex.data.insert(**gramex.service.otp, id=['token'], args={
-            'user': [user], 'email': [email], 'token': [token], 'expire': [expire],
-        })
-        return token
-
-    def pop(self, token):
-        '''Return the row matching the token, and deletes it from the list'''
-        rows = gramex.data.filter(**gramex.service.otp, args={'token': [token]})
-        if len(rows) > 0:
-            row = rows.iloc[0].to_dict()
-            if row['expire'] > time.time():
-                gramex.data.delete(**gramex.service.otp, id=['token'], args={'token': [token]})
-                return row
-        return None
-
-
-def csv_encode(values, *args, **kwargs):
-    '''
-    Encode an array of unicode values into a comma-separated string. All
-    csv.writer parameters are valid.
-    '''
-    buf = io.StringIO()
-    writer = csv.writer(buf, *args, **kwargs)
-    writer.writerow([
-        v if isinstance(v, str) else
-        v.decode('utf-8') if isinstance(v, bytes) else repr(v)
-        for v in values])
-    return buf.getvalue().strip()
