@@ -75,28 +75,15 @@ class FileHandler(BaseHandler):
         structure as :class:`FunctionHandler`, and accepts these keys:
 
         ``function``
-            A string that resolves into any Python function or method (e.g.
-            ``markdown.markdown``). By default, it is called with the file
-            contents as ``function(content)`` and the result is rendered as-is
-            (hence must be a string.)
-
-        ``args``
-            optional positional arguments to be passed to the function. By
-            default, this is just ``['content']`` where ``content`` is the file
-            contents. You can also pass the handler via ``['handler']``, or both
-            of them in any order.
-
-        ``kwargs``:
-            an optional list of keyword arguments to be passed to the function.
-            A value with of ``handler`` and ``content`` is replaced with the
-            RequestHandler and file contents respectively.
+            The expression to return. Example: ``function: mymodule.transform(content, handler)``.
+            ``content`` has the file contents. ``handler`` has the FileHandler object
 
         ``encoding``
-            The encoding to load the file as. If you don't specify an encoding,
-            file contents are passed to ``function`` as a binary string.
+            The encoding to read the file with, e.g. ``utf-8``. If ``None`` (the default), the
+            file is read as bytes, and the transform `function` MUST accept the content as bytes
 
         ``headers``:
-            HTTP headers to set on the response.
+            HTTP headers to set on the response
     :arg string template: ``template="*.html"`` renders all HTML files as Tornado templates.
         ``template=True`` renders all files as Tornado templates (new in Gramex 1.14).
     :arg string sass: ``sass="*.sass"`` renders all SASS files as CSS (new in Gramex 1.66).
@@ -113,8 +100,16 @@ class FileHandler(BaseHandler):
     '''
 
     @classmethod
-    def setup(cls, path, default_filename=None, index=None, index_template=None,
-              headers={}, default={}, **kwargs):
+    def setup(
+        cls,
+        path,
+        default_filename=None,
+        index=None,
+        index_template=None,
+        headers={},
+        default={},
+        **kwargs,
+    ):
         # Convert template: '*.html' into transform: {'*.html': {function: template}}
         # Convert sass: ['*.scss', '*.sass'] into transform: {'*.scss': {function: sass}}
         # Do this before BaseHandler setup so that it can invoke the transforms required
@@ -123,8 +118,9 @@ class FileHandler(BaseHandler):
             if val:
                 # template/sass/...: true is the same as template: '*'
                 val = '*' if val is True else val if isinstance(val, (list, tuple)) else [val]
-                kwargs.setdefault('transform', AttrDict()).update({
-                    v: AttrDict(function=key) for v in val})
+                kwargs.setdefault('transform', AttrDict()).update(
+                    {v: AttrDict(function=key) for v in val}
+                )
         super(FileHandler, cls).setup(**kwargs)
 
         cls.root, cls.pattern = None, None
@@ -160,7 +156,7 @@ class FileHandler(BaseHandler):
         Convert value to a set. If value is already a list, set, tuple, return as is.
         Ensure that the values are non-empty strings.
         '''
-        result = set(value) if isinstance(value, (list, tuple, set)) else set([value])
+        result = set(value) if isinstance(value, (list, tuple, set)) else {value}
         for pattern in result:
             if not pattern:
                 app_log.warning(f'{cls.name}: Ignoring empty pattern "{pattern!r}"')
@@ -257,8 +253,11 @@ class FileHandler(BaseHandler):
             raise HTTPError(FORBIDDEN, f'{self.file} not allowed')
 
         # Display the list of files for directories without a default file
-        if self.path.is_dir() and self.index and not (
-                self.default_filename and self.file.exists()):
+        if (
+            self.path.is_dir()
+            and self.index
+            and not (self.default_filename and self.file.exists())
+        ):
             self.set_header('Content-Type', 'text/html; charset=UTF-8')
             content = []
             file_template = string.Template('<li><a href="$path">$name</a></li>')
@@ -270,10 +269,12 @@ class FileHandler(BaseHandler):
                 else:
                     name_suffix = path_suffix = ''
                 path = str(path.relative_to(self.path))
-                content.append(file_template.substitute(
-                    path=path + path_suffix,
-                    name=path + name_suffix,
-                ))
+                content.append(
+                    file_template.substitute(
+                        path=path + path_suffix,
+                        name=path + name_suffix,
+                    )
+                )
             content.append('</ul>')
             try:
                 tmpl = gramex.cache.open(self.index_template, _tmpl_opener)
