@@ -51,14 +51,13 @@ def get_model(mclass: str, model_params: dict, **kwargs) -> ml.AbstractModel:
 
 
 class MLHandler(FormHandler):
-
     @classmethod
     def setup(cls, data=None, model={}, config_dir='', template=DEFAULT_TEMPLATE, **kwargs):
         if not config_dir:
-            config_dir = op.join(gramex.config.variables['GRAMEXDATA'], 'apps', 'mlhandler',
-                                 slugify(cls.name))
+            config_dir = op.join(
+                gramex.config.variables['GRAMEXDATA'], 'apps', 'mlhandler', slugify(cls.name)
+            )
         cls.store = ml.ModelStore(config_dir)
-
         cls.template = template
         super(MLHandler, cls).setup(**kwargs)
         index_col = None
@@ -68,7 +67,9 @@ class MLHandler(FormHandler):
                 data['transform'] = build_transform(
                     {'function': data['transform']},
                     vars={'data': None, 'handler': None},
-                    filename='MLHandler:data', iter=False)
+                    filename='MLHandler:data',
+                    iter=False,
+                )
                 cls._built_transform = staticmethod(data['transform'])
             else:
                 cls._built_transform = staticmethod(lambda x: x)
@@ -96,12 +97,17 @@ class MLHandler(FormHandler):
         cls.store.dump('class', mclass)
         cls.store.dump('params', model_params)
         if op.exists(cls.store.model_path):  # If the pkl exists, load it
-            cls.model = get_model(cls.store.model_path, {})
+            if op.isdir(cls.store.model_path):
+                mclass, wrapper = ml.search_modelclass(mclass)
+                cls.model = locate(wrapper).from_disk(cls.store.model_path, mclass)
+            else:
+                cls.model = get_model(cls.store.model_path, {})
         elif data is not None:
             data = cls._filtercols(data)
             data = cls._filterrows(data)
-            cls.model = get_model(mclass, model_params, data=data, cats=cats,
-                                  nums=nums, target_col=target_col)
+            cls.model = get_model(
+                mclass, model_params, data=data, cats=cats, nums=nums, target_col=target_col
+            )
             # train the model
             if issubclass(cls.model.__class__, TransformerMixin):
                 target = None
@@ -110,9 +116,12 @@ class MLHandler(FormHandler):
                 target = data[target_col]
                 train = data.drop([target_col], axis=1)
             gramex.service.threadpool.submit(
-                cls.model.fit, train, target,
-                model_path=cls.store.model_path, name=cls.name,
-                **cls.store.model_kwargs()
+                cls.model.fit,
+                train,
+                target,
+                model_path=cls.store.model_path,
+                name=cls.name,
+                **cls.store.model_kwargs(),
             )
 
     def _parse_multipart_form_data(self):
@@ -130,7 +139,7 @@ class MLHandler(FormHandler):
         return pd.concat(dfs, axis=0)
 
     def _parse_application_json(self):
-        return pd.read_json(self.request.body.decode('utf8'))
+        return pd.read_json(self.request.body.decode('utf-8'))
 
     def _parse_data(self, _cache=True, append=False):
         header = self.request.headers.get('Content-Type', '').split(';')[0]
@@ -164,11 +173,16 @@ class MLHandler(FormHandler):
 
     @classmethod
     def _filterrows(cls, data, **kwargs):
-        for method in 'dropna drop_duplicates'.split():
+        for method in ('dropna', 'drop_duplicates'):
             action = kwargs.get(method, cls.store.load(method, True))
             if action:
                 subset = action if isinstance(action, list) else None
-                data = getattr(data, method)(subset=subset)
+                try:
+                    data = getattr(data, method)(subset=subset)
+                except TypeError as exc:
+                    # The label column for an NER dataset is a nested list.
+                    # Can't do drop_duplicates on that.
+                    app_log.warning(exc)
         return data
 
     def _transform(self, data, **kwargs):
@@ -180,12 +194,12 @@ class MLHandler(FormHandler):
         return data
 
     def _predict(self, data=None, score_col=''):
+        self._check_model_path()
         metric = self.get_argument('_metric', False)
         if metric:
             scorer = get_scorer(metric)
         if data is None:
             data = self._parse_data(False)
-        self.model = get_model(self.store.model_path, {})
         data = self._transform(data, drop_duplicates=False)
         try:
             target = data.pop(score_col)
@@ -203,7 +217,8 @@ class MLHandler(FormHandler):
 
     def _check_model_path(self):
         try:
-            self.model = get_model(self.store.model_path, {})
+            klass, wrapper = ml.search_modelclass(self.store.load('class'))
+            self.model = locate(wrapper).from_disk(self.store.model_path, klass=klass)
         except FileNotFoundError:
             raise HTTPError(NOT_FOUND, f'No model found at {self.store.model_path}')
 
@@ -221,10 +236,7 @@ class MLHandler(FormHandler):
     def get(self, *path_args, **path_kwargs):
         self.set_header('Content-Type', 'application/json')
         if '_params' in self.args:
-            params = {
-                'opts': self.store.load('transform'),
-                'params': self.store.load('model')
-            }
+            params = {'opts': self.store.load('transform'), 'params': self.store.load('model')}
             try:
                 attrs = get_model(self.store.model_path, {}).get_attributes()
             except (AttributeError, ImportError, FileNotFoundError):
@@ -237,8 +249,10 @@ class MLHandler(FormHandler):
             self._check_model_path()
             if '_download' in self.args:
                 self.set_header('Content-Type', 'application/octet-strem')
-                self.set_header('Content-Disposition',
-                                f'attachment; filename={op.basename(self.store.model_path)}')
+                self.set_header(
+                    'Content-Disposition',
+                    f'attachment; filename={op.basename(self.store.model_path)}',
+                )
                 with open(self.store.model_path, 'rb') as fout:
                     self.write(fout.read())
             elif '_model' in self.args:
@@ -255,8 +269,7 @@ class MLHandler(FormHandler):
                     data = []
                 if len(data) > 0:
                     data = data.drop([self.store.load('target_col')], axis=1, errors='ignore')
-                    prediction = yield gramex.service.threadpool.submit(
-                        self._predict, data)
+                    prediction = yield gramex.service.threadpool.submit(self._predict, data)
                     self.write(json.dumps(prediction, indent=2, cls=CustomJSONEncoder))
                 else:
                     self.set_header('Content-Type', 'text/html')
@@ -274,9 +287,12 @@ class MLHandler(FormHandler):
         data = self._filtercols(data)
         data = self._filterrows(data)
         self.model = get_model(
-            self.store.load('class'), self.store.load('params'),
-            data=data, target_col=target_col,
-            nums=self.store.load('nums'), cats=self.store.load('cats')
+            self.store.load('class'),
+            self.store.load('params'),
+            data=data,
+            target_col=target_col,
+            nums=self.store.load('nums'),
+            cats=self.store.load('cats'),
         )
         if not isinstance(self.model, ml.SklearnTransformer):
             target = data[target_col]
@@ -342,3 +358,43 @@ class MLHandler(FormHandler):
                 getattr(self, f'_delete_{item}')()
             except AttributeError:
                 raise HTTPError(BAD_REQUEST, f'Cannot delete {item}.')
+
+
+class MLPredictor(FormHandler):
+    @classmethod
+    def setup(cls, config_dir, data, target_col=None, columns=None, **kwargs):
+        cls.config_dir = config_dir
+        cls.cols = columns
+        cls.target_col = target_col
+        kwargs.update(data)
+        super(MLPredictor, cls).setup(**kwargs)
+
+    def _filtercols(self, data):
+        """Filter columns from the input dataframe, as follows:
+
+        1. If columns are specified, use only those columns and ignore everything else.
+        2. Otherwise fall back on config_dir/config.json
+
+        Note: From sklearn 1.x onwards, ColumnTransformer (which is the default
+        preprocessor used in MLHandler) stores the names of columns.
+        When Gramex supports it, we will not need to store column information in config.json
+        """
+        if self.cols is not None:
+            return data[self.cols]
+        store = ml.ModelStore(self.config_dir)
+        include = store.load('include')
+        if len(include):
+            return data[include]
+        exclude = store.load('exclude')
+        target = store.load('target_col', False)
+        if target and target not in exclude:
+            exclude.append(target)
+        return data.drop(exclude, axis=1)
+
+    def modify_all(self, data=None, key=None, handler=None):
+        model = cache.open(op.join(self.config_dir, "model.pkl"), joblib.load)
+        df = self._filtercols(data['data'])
+        store = ml.ModelStore(self.config_dir)
+        target = self.target_col or store.load('target_col', None) or 'prediction'
+        df[target] = model.predict(df)
+        return {'data': df}
