@@ -64,6 +64,7 @@ class ProcessHandler(BaseHandler):
     .. _Popen: https://docs.python.org/3/library/subprocess.html#subprocess.Popen
 
     '''
+
     @classmethod
     def setup(cls, args, shell=False, cwd=None, buffer=0, headers={}, **kwargs):
         super(ProcessHandler, cls).setup(**kwargs)
@@ -125,7 +126,8 @@ class ProcessHandler(BaseHandler):
         proc = Subprocess(
             self.cmdargs,
             # NOTE: developer should sanitize args if shell=True
-            shell=self.shell,  # nosec: developer-initiated
+            # B604 any_other_function_with_shell_equals_true
+            shell=self.shell,  # nosec B604
             cwd=self.cwd,
             stream_stdout=self.stream_stdout,
             stream_stderr=self.stream_stderr,
@@ -146,6 +148,6 @@ class ProcessHandler(BaseHandler):
 
     def on_finish(self):
         '''Close all open handles after the request has finished'''
-        for target, handle in self.handles.items():
+        for handle in self.handles.values():
             handle.close()
         super(ProcessHandler, self).on_finish()
