@@ -1,6 +1,7 @@
 '''
 Defines command line services to install, setup and run apps.
 '''
+import contextlib
 import io
 import os
 import sys
@@ -12,10 +13,12 @@ import string
 import shutil
 import datetime
 import requests
-from shutilwhich import which
+from shutil import which
 from pathlib import Path
-# subprocess is safe since it runs developer-initiated commands
-from subprocess import Popen, check_output, CalledProcessError      # nosec: developer-initiated
+from collections import Counter
+
+# B404:import_subprocess only developers can access this, not users
+from subprocess import Popen, check_output, CalledProcessError  # nosec B404
 from orderedattrdict import AttrDict
 from orderedattrdict.yamlutils import AttrDictYAMLLoader
 from zipfile import ZipFile
@@ -153,13 +156,17 @@ license: |
     gramex license                  # Show Gramex license
     gramex license accept           # Accept Gramex license
     gramex license reject           # Reject Gramex license
+
+features: |
+    gramex features                 # Show Gramex features used in current project
 '''
-# yaml.load is safe since it only reads the string above, not user-created content
-usage = yaml.load(usage, Loader=AttrDictYAMLLoader)     # nosec: frozen input
+# B506:yaml_load yaml.load is safe since it only reads the string above, not user-created content
+usage = yaml.load(usage, Loader=AttrDictYAMLLoader)  # nosec B506
 
 
 class TryAgainError(Exception):
     '''If shutil.rmtree fails, and we've fixed the problem, raise this to try again'''
+
     pass
 
 
@@ -167,14 +174,19 @@ try:
     WindowsError
 except NameError:
     # On non-Windows systems, _ensure_remove just raises the exception
-    def _ensure_remove(remove, path, exc_info):     # noqa -- redefine function
+    def _ensure_remove(remove, path, exc_info):
         raise exc_info[1]
+
 else:
     # On Windows systems, try harder
     def _ensure_remove(func, path, exc_info):
         '''onerror callback for rmtree that tries hard to delete files'''
         if issubclass(exc_info[0], WindowsError):
-            import winerror
+            winerror = AttrDict(
+                ERROR_PATH_NOT_FOUND=3,
+                ERROR_ACCESS_DENIED=5,
+                ERROR_SHARING_VIOLATION=32,
+            )
             # Delete read-only files
             # https://bugs.python.org/issue19643
             # https://bugs.python.org/msg218021
@@ -188,17 +200,18 @@ else:
                 delays = [0.001, 0.002, 0.005, 0.01, 0.02, 0.05, 0.1, 0.2, 0.5, 1.0]
                 for delay in delays:
                     time.sleep(delay)
-                    try:
+                    with contextlib.suppress(WindowsError):
                         return os.remove(path)
-                    except WindowsError:
-                        pass
             # npm creates windows shortcuts that shutil.rmtree cannot delete.
             # os.listdir/scandir fails with a PATH_NOT_FOUND.
             # Delete these using win32com and try again.
-            elif (exc_info[1].winerror == winerror.ERROR_PATH_NOT_FOUND and
-                  func in {os.listdir, os.scandir}):
+            elif exc_info[1].winerror == winerror.ERROR_PATH_NOT_FOUND and func in {
+                os.listdir,
+                os.scandir,
+            }:
                 app_log.error(f'Cannot delete {path}')
                 from win32com.shell import shell, shellcon  # type:ignore
+
                 options = shellcon.FOF_NOCONFIRMATION | shellcon.FOF_NOERRORUI
                 code, err = shell.SHFileOperation((0, shellcon.FO_DELETE, path, None, options))
                 if code == 0:
@@ -207,7 +220,7 @@ else:
 
 
 def _try_remove(target, retries=100, delay=0.05, func=shutil.rmtree, **kwargs):
-    for count in range(retries):
+    for _index in range(retries):
         try:
             func(target, **kwargs)
         except TryAgainError:
@@ -229,8 +242,9 @@ def safe_rmtree(target, retries=100, delay=0.05, gramexdata=True):
     if not os.path.exists(target):
         return True
     # TODO: check case insensitive in Windows, but case sensitive on other OS
-    func, kwargs = (shutil.rmtree, {'onerror': _ensure_remove}) if \
-        os.path.isdir(target) else (os.remove, {})
+    func, kwargs = (
+        (shutil.rmtree, {'onerror': _ensure_remove}) if os.path.isdir(target) else (os.remove, {})
+    )
     if gramexdata:
         if target.lower().startswith(variables['GRAMEXDATA'].lower()):
             # Try multiple times to recover from errors, since we have no way of
@@ -276,9 +290,8 @@ def run_install(config):
         if os.path.exists(target):
             url = os.path.abspath(url).lower().rstrip(os.sep)
             target = os.path.abspath(target).lower().rstrip(os.sep)
-            if url != target:
-                if not safe_rmtree(target):
-                    return
+            if url != target and not safe_rmtree(target):
+                return
         if url != target:
             shutil.copytree(url, target)
             app_log.info(f'Copied {url} into {target}')
@@ -320,13 +333,14 @@ def run_command(config):
         appcmd = shlex.split(appcmd)
     # If the app is a Cygwin app, TARGET should be a Cygwin path too.
     target = config.target
-    cygcheck, cygpath, kwargs = which('cygcheck'), which('cygpath'), {'universal_newlines': True}
-    if cygcheck is not None and cygpath is not None:
+    cygwin, cygpath, kwargs = which('cygcheck'), which('cygpath'), {'universal_newlines': True}
+    if cygwin is not None and cygpath is not None:
         # subprocess.check_output is safe here since these are developer-initiated
-        app_path = check_output([cygpath, '-au', which(appcmd[0])], **kwargs).strip()   # nosec
-        is_cygwin_app = check_output([cygcheck, '-f', app_path], **kwargs).strip()      # nosec
+        # B404:import_subprocess check_output is safe here since these are developer-initiated
+        path = check_output([cygpath, '-au', which(appcmd[0])], **kwargs).strip()  # nosec 404
+        is_cygwin_app = check_output([cygwin, '-f', path], **kwargs).strip()  # nosec 404
         if is_cygwin_app:
-            target = check_output([cygpath, '-au', target], **kwargs).strip()           # nosec
+            target = check_output([cygpath, '-au', target], **kwargs).strip()  # nosec 404
     # Replace TARGET with the actual target
     if 'TARGET' in appcmd:
         appcmd = [target if arg == 'TARGET' else arg for arg in appcmd]
@@ -336,7 +350,8 @@ def run_command(config):
     if not safe_rmtree(config.target):
         app_log.error(f'Cannot delete target {config.target}. Aborting installation')
         return
-    proc = Popen(appcmd, bufsize=-1, **kwargs)      # nosec: developer-initiated
+    # B603:subprocess_without_shell_equals_true is safe since this is developer-initiated
+    proc = Popen(appcmd, bufsize=-1, **kwargs)  # nosec 603
     proc.communicate()
     return proc.returncode
 
@@ -354,7 +369,7 @@ setup_paths = [
         {'file': 'yarn.lock', 'exe': 'yarn', 'cmd': '"{exe}" install --prefer-offline'},
         {'file': 'package.json', 'exe': 'npm', 'cmd': '"{exe}" install'},
         {'file': 'package.json', 'exe': 'yarn', 'cmd': '"{exe}" install --prefer-offline'},
-    ]
+    ],
 ]
 
 
@@ -446,8 +461,7 @@ def flatten_config(config, base=None):
     for key, value in config.items():
         keystr = key if base is None else base + '.' + key
         if hasattr(value, 'items'):
-            for sub in flatten_config(value, keystr):
-                yield sub
+            yield from flatten_config(value, keystr)
         else:
             yield keystr, value
 
@@ -456,9 +470,8 @@ def show_usage(command):
     apps = (+apps_config).keys()
     return 'gramex {command}\n\n{desc}'.format(
         command=command,
-        desc=usage[command].strip().format(
-            apps='\n'.join('- ' + app for app in sorted(apps))
-        ))
+        desc=usage[command].strip().format(apps='\n'.join('- ' + app for app in sorted(apps))),
+    )
 
 
 def install(args, kwargs):
@@ -543,24 +556,33 @@ def run(args, kwargs):
         gramex.paths['base'] = Path('.')
         # If we run with updated parameters, save for next run under the .run config
         run_config = app_config.setdefault('run', {})
-        for key, val in kwargs.items():
+        for key in kwargs:
             if key not in app_keys:
                 run_config[key] = app_config.pop(key)
         save_user_config(appname, app_config)
         # Tell the user what configs are used
         cline = ' '.join('--%s=%s' % arg for arg in flatten_config(app_config.get('run', {})))
-        app_log.info('Gramex %s | %s %s | %s | Python %s', gramex.__version__, appname, cline,
-                     os.getcwd(), sys.version.replace('\n', ' '))
+        app_log.info(
+            'Gramex %s | %s %s | %s | Python %s',
+            gramex.__version__,
+            appname,
+            cline,
+            os.getcwd(),
+            sys.version.replace('\n', ' '),
+        )
         gramex.init(args=AttrDict(app=app_config['run']))
     elif appname in apps_config['user']:
         # The user configuration has a wrong path. Inform user
-        app_log.error(f'{appname}: no target path {app_config.target}. '
-                      f'Run "gramex uninstall {appname}" and try again.', )
+        app_log.error(
+            f'{appname}: no target path {app_config.target}. '
+            f'Run "gramex uninstall {appname}" and try again.',
+        )
     else:
         app_log.error(f'{appname}: no target path {app_config.target}')
 
 
 def service(args, kwargs):
+    '''Install, remove, start or stop Gramex as a Windows service.'''
     try:
         import gramex.winservice
     except ImportError:
@@ -575,7 +597,8 @@ def service(args, kwargs):
 def _check_output(cmd, default=b'', **kwargs):
     '''Run cmd and return output. Return default in case the command fails'''
     try:
-        return check_output(shlex.split(cmd), **kwargs).strip()     # nosec: developer-initiated
+        # B603:subprocess_without_shell_equals_true is safe since this is developer-initiated
+        return check_output(shlex.split(cmd), **kwargs).strip()  # nosec B603
     # OSError is raised if the cmd is not found.
     # CalledProcessError is raised if the cmd returns an error.
     except (OSError, CalledProcessError):
@@ -586,7 +609,8 @@ def _run_console(cmd, **kwargs):
     '''Run cmd and pipe output to console. Log and raise error if cmd is not found'''
     cmd = shlex.split(cmd)
     try:
-        proc = Popen(cmd, bufsize=-1, universal_newlines=True, **kwargs)
+        # B603:subprocess_without_shell_equals_true is safe since this is developer-initiated
+        proc = Popen(cmd, bufsize=-1, universal_newlines=True, **kwargs)  # nosec B603
     except OSError:
         app_log.error(f'Cannot find command: {cmd[0]}')
         raise
@@ -608,7 +632,7 @@ def init(args, kwargs):
         args.append('default')
     source_dir = os.path.join(variables['GRAMEXPATH'], 'apps', 'init', args[0])
     if not os.path.exists(source_dir):
-        app_log.error(f'Unknown init template {args[0]}')
+        app_log.error(f'Unknown init template: {args[0]} under {source_dir}')
 
     kwargs.setdefault('target', os.getcwd())
     app_log.info(f'Initializing Gramex project at {kwargs.target}')
@@ -626,10 +650,8 @@ def init(args, kwargs):
     data['appname'] = appname
 
     # Create a git repo. But if git fails, do not stop. Continue with the rest.
-    try:
+    with contextlib.suppress(OSError):
         _run_console('git init')
-    except OSError:
-        pass
     # Install Git LFS if available. Set git_lfs=None if it fails, so .gitignore ignores assets/**
     data['git_lfs'] = which('git-lfs')
     if data['git_lfs']:
@@ -676,8 +698,7 @@ email:
     type: gmail
     email: $GRAMEXMAILUSER
     password: $GRAMEXMAILPASSWORD
-    # Uncomment the next line to test the application without sending mails
-    # stub: log
+    # Add stub: log to test the application without sending mails
 
 # See https://gramener.com/gramex/guide/alert/
 alert:
@@ -729,6 +750,7 @@ def mail(args, kwargs):
         return
 
     from gramex.services import email as setup_email, create_alert
+
     alert_conf = conf.get('alert', {})
     email_conf = conf.get('email', {})
     setup_email(email_conf)
@@ -754,3 +776,174 @@ def license(args, kwargs):
         gramex.license.reject()
     else:
         app_log.error(f'Invalid command license {args[0]}')
+
+
+def features(args, kwargs) -> dict:
+    """
+    This function extracts feature information from a gramex.yaml file and returns
+    a dictionary with a Pandas DataFrame containing information about the features
+    used in the gramex application.
+
+    Args:
+        args: This function doesn't use any arguments.
+        kwargs: A dictionary that can contain a 'conf' key that specifies the path
+            to the gramex.yaml file. If 'conf' is not specified, the function will
+            use the current working directory.
+
+    Returns:
+        A dictionary containing a Pandas DataFrame with the following columns:
+            - type: The type of feature (e.g., 'MS' for a URL handler)
+            - feature: The name of the feature (e.g., the name of the URL handler)
+            - count: The number of times the feature is used in the gramex app.
+    """
+    import pandas as pd
+
+    project_yaml = _gramex_yaml_path(os.getcwd() if len(args) == 0 else args[0], kwargs)
+    if project_yaml is None:
+        return
+
+    # Load the feature mapping from cache
+    features = gramex.cache.open("gramexfeatures.csv", rel=True).set_index(["type", "name"])[
+        "feature"
+    ]
+    # Load the gramex.yaml configuration
+    base = gramex.config.PathConfig(os.path.join(os.path.dirname(gramex.__file__), 'gramex.yaml'))
+    proj_features = Counter({(type, feature): 0 for ((type, _name), feature) in features.items()})
+    try:
+        app = gramex.config.PathConfig(project_yaml)
+        conf = +gramex.config.ChainConfig([('base', base), ('app', app)])
+    except Exception as e:  # noqa: B902 capture load errors as a "feature"
+        app_log.exception(str(e))
+        return
+
+    # If the configuration is not a dictionary, return an empty result
+    if not isinstance(conf, dict):
+        return
+
+    # List the features used. TODO: Improve this using configuration
+    for url in conf.get('url', {}).values():
+        if not isinstance(url, dict):
+            continue
+        name = url.get('handler', '').replace('gramex.handlers.', '')
+        if name in features['MS']:
+            proj_features[('MS', features['MS'][name])] += 1
+        else:
+            proj_features[('ERR-MS', name)] += 1
+
+        if 'cache' in url:
+            proj_features[('KWARG', 'cache')] += 1
+
+        url_kwargs = url.get('kwargs', {})
+        for key in url_kwargs:
+            if key not in features['KWARG']:
+                continue
+            proj_features[('KWARG', features['KWARG'][key])] += 1
+
+    # List the services used
+    for service in conf.keys():
+        # If the service is conditional, e.g. `log if condition`, take just the first part
+        name = service.split()[0]
+        if name in features['SVC']:
+            proj_features[('SVC', features['SVC'][name])] += 1
+
+    # Create features table
+
+    proj_features = pd.DataFrame(proj_features.values(), index=proj_features.keys()).reset_index()
+    proj_features.columns = ['type', 'feature', 'count']
+
+    return proj_features
+
+
+def complexity(args, kwargs) -> dict:
+    # Calulate cyclomatic complexity of the project
+    import ast
+    import fnmatch
+    import mccabe
+    import pandas as pd
+    import re
+
+    # B404:import_subprocess only used for internal Gramex scripts
+    from subprocess import check_output  # nosec B404
+
+    project_path = os.getcwd() if len(args) == 0 else args[0]
+    project_yaml = _gramex_yaml_path(project_path, kwargs)
+    if project_yaml is None:
+        return
+
+    def walk(node: dict, parents: tuple = ()):
+        for key, value in node.items():
+            new_key = parents + (str(key),)
+            if hasattr(value, 'items'):
+                yield new_key, None
+                yield from walk(value, new_key)
+            else:
+                yield new_key, value
+
+    py_complexity = 0
+    relevent_file_matcher = re.compile(r'^((?!(site-packages|node_modules)).)*\.py$')
+
+    for root, _dirs, files in os.walk(project_path):
+        for filename in files:
+            path: str = os.path.join(root, filename)
+            if not relevent_file_matcher.match(path):
+                continue
+            rel_path: str = os.path.relpath(path, project_path)
+            with open(path, 'rb') as handle:
+                code = handle.read()
+            try:
+                tree = compile(code, rel_path, 'exec', ast.PyCF_ONLY_AST)
+            except SyntaxError as e:
+                app_log.exception('SYNTAXERROR: %s %s', path, e)
+                continue
+            visitor = mccabe.PathGraphingAstVisitor()
+            visitor.preorder(tree, visitor)
+            for node in visitor.graphs.values():
+                py_complexity += node.complexity()
+
+    base = gramex.config.PathConfig(os.path.join(os.path.dirname(gramex.__file__), 'gramex.yaml'))
+    try:
+        app = gramex.config.PathConfig(project_yaml)
+        conf = +gramex.config.ChainConfig([('base', base), ('app', app)])
+    except Exception as e:  # noqa: B902 capture load errors as a "feature"
+        app_log.exception(str(e))
+        return
+    yamlpaths = {'.'.join(key): val for key, val in walk(conf)}
+    used = set()
+    gramexsize = gramex.cache.open('gramexsize.csv', rel=True)
+    for _index, gramex_code in gramexsize.iterrows():
+        # TODO: fnmatch.filter() is the slowest part. Optimize it
+        if '=' not in gramex_code['yamlpath']:
+            if fnmatch.filter(yamlpaths.keys(), gramex_code['yamlpath']):
+                used.add(gramex_code['codepath'])
+        else:
+            yaml_key, yaml_value = gramex_code['yamlpath'].split('=', 1)
+            matches = fnmatch.filter(yamlpaths.keys(), yaml_key)
+            for key in matches:
+                if yamlpaths[key] in yaml_value.split('|'):
+                    used.add(gramex_code['codepath'])
+    gramex_complexity = gramexsize.set_index('codepath')['complexity'][list(used)].sum()
+
+    # Calculate JS complexity
+    # B602:subprocess_popen_with_shell_equals_true and
+    # B607:start_process_with_partial_path are safe to skip since this is a Gramex internal cmd
+    output = check_output(['npx', '@gramex/escomplexity'], cwd=project_path, shell=True)  # nosec
+    es_complexity = int(output.decode('utf-8').split('\n')[-2].strip())
+    return pd.DataFrame(
+        {
+            'py': [py_complexity],
+            'js': [es_complexity],
+            'gramex': [gramex_complexity],
+        }
+    )
+
+
+def _gramex_yaml_path(folder, kwargs):
+    project_yaml = os.path.join(folder, 'gramex.yaml')
+    # Get config file location
+    if 'conf' in kwargs:
+        return kwargs.conf
+    elif os.path.exists(project_yaml):
+        return project_yaml
+    else:
+        app_log.error(f'Missing {project_yaml}')
+        return
