@@ -1,4 +1,5 @@
-import pickle       # nosec: we only pickle Gramex internal objects
+# B403:import_public we only pickle Gramex internal objects
+import pickle  # nosec B403
 from redis import StrictRedis
 
 
@@ -6,11 +7,11 @@ def get_redis(path: str, **kwargs):
     host, port, db, redis_kwargs = 'localhost', 6379, 0, {}
     if isinstance(path, str):
         parts = path.split(':')
-        if len(parts):
+        if parts:
             host = parts.pop(0)
-        if len(parts):
+        if parts:
             port = int(parts.pop(0))
-        if len(parts):
+        if parts:
             db = int(parts.pop(0))
         redis_kwargs = dict(part.split('=', 1) for part in parts)
     for key, val in kwargs.items():
@@ -18,7 +19,7 @@ def get_redis(path: str, **kwargs):
     return StrictRedis(host=host, port=port, db=db, **redis_kwargs)
 
 
-class RedisCache():
+class RedisCache:
     '''
     LRU Cache that stores data in a Redis database. Typical usage::
 
@@ -39,6 +40,7 @@ class RedisCache():
     Both Keys and Values are stored as pickle dump.
     This is an approximate LRU implementation. Read more here.(https://redis.io/topics/lru-cache)
     '''
+
     def __init__(self, path=None, maxsize=None, *args, **kwargs):
         self.store = get_redis(path, decode_responses=False)
         self.size = 0
@@ -51,7 +53,8 @@ class RedisCache():
     def __getitem__(self, key):
         key = pickle.dumps(key, pickle.HIGHEST_PROTOCOL)
         result = self.store.get(key)
-        return None if result is None else pickle.loads(result)     # nosec: frozen input
+        # B301:pickle key is set by developers and safe to pickle
+        return None if result is None else pickle.loads(result)  # nosec B301
 
     def __setitem__(self, key, value, expire=None):
         key = pickle.dumps(key, pickle.HIGHEST_PROTOCOL)
@@ -67,7 +70,8 @@ class RedisCache():
     def __iter__(self):
         for key in self.store.scan_iter():
             try:
-                yield pickle.loads(key)     # nosec: key is safe
+                # B301:pickle key is set by developers and safe to pickle
+                yield pickle.loads(key)  # nosec B301
             except pickle.UnpicklingError:
                 # If redis already has keys created by other apps, yield them as-is
                 yield key
