@@ -41,14 +41,18 @@ class GramexService(win32serviceutil.ServiceFramework):
     def SvcStop(self):
         self.ReportServiceStatus(win32service.SERVICE_STOP_PENDING)
         import gramex
+
         gramex.shutdown()
         win32event.SetEvent(self.hWaitStop)
         cwd = win32serviceutil.GetServiceCustomOption(self, 'cwd')
         exe = win32serviceutil.GetServiceCustomOption(self, 'exe')
         gramexpath = win32serviceutil.GetServiceCustomOption(self, 'py')
         msg = f'\nPath: {cwd}. Python: {exe}. Gramex: {gramexpath}'
-        servicemanager.LogMsg(servicemanager.EVENTLOG_INFORMATION_TYPE,
-                              servicemanager.PYS_SERVICE_STOPPED, (self._svc_name_, msg))
+        servicemanager.LogMsg(
+            servicemanager.EVENTLOG_INFORMATION_TYPE,
+            servicemanager.PYS_SERVICE_STOPPED,
+            (self._svc_name_, msg),
+        )
 
     def SvcDoRun(self):
         # Change to cwd registry option
@@ -66,23 +70,29 @@ class GramexService(win32serviceutil.ServiceFramework):
         servicelogfile = os.path.join(cwd, 'service.log')
         sys.stdout = sys.stderr = io.open(servicelogfile, 'a', encoding='utf-8')
         # Log start of service
-        servicemanager.LogMsg(typ, servicemanager.PYS_SERVICE_STARTING, (
-            self._svc_name_, f'\n{err}Path: {cwd}. Python: {exe}. Gramex: {gramexpath}'))
+        servicemanager.LogMsg(
+            typ,
+            servicemanager.PYS_SERVICE_STARTING,
+            (self._svc_name_, f'\n{err}Path: {cwd}. Python: {exe}. Gramex: {gramexpath}'),
+        )
         # See https://github.com/tornadoweb/tornado/issues/2608
-        if sys.version_info.minor >= 8:
+        if sys.version_info >= (3, 8):
             import asyncio
+
             asyncio.set_event_loop_policy(asyncio.WindowsSelectorEventLoopPolicy())
         # On Conda, we can't import NumPy because Library\bin is not in PATH. Add it
         try:
-            import numpy    # noqa
+            import numpy  # noqa
         except ImportError:
             import re
+
             conda_path = re.sub(r'\\lib\\site-packages\\win32\\.*', '', sys.executable)
             os.environ['PATH'] = f'{conda_path}\\Library\\bin;{os.environ["PATH"]}'
         # Run Gramex
         try:
             port = self._svc_port_
             import gramex
+
             gramex.commandline(None if port is None else [f'--listen.port={port}'])
         except Exception:
             typ = servicemanager.EVENTLOG_ERROR_TYPE
@@ -93,6 +103,7 @@ class GramexService(win32serviceutil.ServiceFramework):
     def setup(cls, cmd, user=None, password=None, startup='manual', cwd=None, wait=0):
         import gramex
         from gramex.config import app_log
+
         name, service_name = cls._svc_display_name_, cls._svc_name_
         port = getattr(cls, '_svc_port_', None)
         if cwd is None:
@@ -105,16 +116,28 @@ class GramexService(win32serviceutil.ServiceFramework):
         running = win32service.SERVICE_RUNNING
         if cmd[0] == 'install':
             win32serviceutil.InstallService(
-                service_class, service_name, displayName=name, description=cls._svc_description_,
-                startType=startup, userName=user, password=password)
+                service_class,
+                service_name,
+                displayName=name,
+                description=cls._svc_description_,
+                startType=startup,
+                userName=user,
+                password=password,
+            )
             win32serviceutil.SetServiceCustomOption(cls._svc_name_, 'cwd', cwd)
             win32serviceutil.SetServiceCustomOption(cls._svc_name_, 'exe', exe)
             win32serviceutil.SetServiceCustomOption(cls._svc_name_, 'py', gramexpath)
             app_log.info(f'Installed service. {name} will run from {path}')
         elif cmd[0] in {'update', 'change'}:
             win32serviceutil.ChangeServiceConfig(
-                service_class, service_name, displayName=name, description=cls._svc_description_,
-                startType=startup, userName=user, password=password)
+                service_class,
+                service_name,
+                displayName=name,
+                description=cls._svc_description_,
+                startType=startup,
+                userName=user,
+                password=password,
+            )
             win32serviceutil.SetServiceCustomOption(cls._svc_name_, 'cwd', cwd)
             win32serviceutil.SetServiceCustomOption(cls._svc_name_, 'exe', exe)
             win32serviceutil.SetServiceCustomOption(cls._svc_name_, 'py', gramexpath)
