@@ -8,8 +8,13 @@ from shutil import rmtree
 from mimetypes import guess_type
 from tornado.web import create_signed_value
 from gramex import conf
-from gramex.http import (OK, BAD_REQUEST, NOT_FOUND, REQUEST_ENTITY_TOO_LARGE,
-                         UNSUPPORTED_MEDIA_TYPE)
+from gramex.http import (
+    OK,
+    BAD_REQUEST,
+    NOT_FOUND,
+    REQUEST_ENTITY_TOO_LARGE,
+    UNSUPPORTED_MEDIA_TYPE,
+)
 from gramex.install import _ensure_remove
 from nose.tools import eq_, ok_
 from . import server, TestGramex, afe
@@ -28,7 +33,7 @@ class TestDriveHandler(TestGramex):
         for f in fileinfo:
             info = (f.get('name', f['file']), open(f['file'], 'rb'))
             if 'mime' in f:
-                info += (f['mime'], )
+                info += (f['mime'],)
             files.append(('file', info))
             for field in ('tag', 'cat'):
                 if field in f:
@@ -39,6 +44,9 @@ class TestDriveHandler(TestGramex):
             headers['X-Gramex-User'] = create_signed_value(secret, 'user', json.dumps(user))
         r = requests.post(self.url, files=files, data=data, headers=headers)
         eq_(r.status_code, code, '%s: code %d != %d' % (self.url, r.status_code, code))
+        if code is OK:
+            paths_exist = json.loads(r.headers['Paths-Exist'])
+            eq_(paths_exist, {f.get('name', os.path.basename(f['file'])): True for f in fileinfo})
         data = gramex.data.filter(self.con, table='drive')
         data = data.sort_values('id').tail(len(files))
         if not check:
@@ -75,43 +83,49 @@ class TestDriveHandler(TestGramex):
         for col in cols:
             ok_(col in data.columns, col)
 
-        self.check_upload(dict(file='userdata.csv'))
+        self.check_upload({'file': 'userdata.csv'})
         # If upload filehame has a path, only the basename is considered
         # Extension is stored in lowercase even if filename is in uppercase
-        self.check_upload(dict(file='dir/image.JPG'))
+        self.check_upload({'file': 'dir/image.JPG'})
 
         # If filename is repeated, it's extended randomly
-        data = self.check_upload(dict(file='userdata.csv'))
+        data = self.check_upload({'file': 'userdata.csv'})
         path = data.path.iloc[0]
         ok_(path != 'userdata.csv')
         ok_(path.startswith('userdata'))
         ok_(path.endswith('.csv'))
 
         # If filename has weird characters, it's hyphenated
-        data = self.check_upload(dict(file='userdata.csv', name='β x.csv'))
+        data = self.check_upload({'file': 'userdata.csv', 'name': 'β x.csv'})
         eq_(data.path.iloc[0], 'b-x.csv')
 
         # If content-type is available, it's used. Else it's guessed
-        data = self.check_upload(dict(file='userdata.csv', mime='text/plain'))
+        data = self.check_upload({'file': 'userdata.csv', 'mime': 'text/plain'})
         eq_(data.mime.iloc[0], 'text/plain')
-        data = self.check_upload(dict(file='userdata.csv'))
+        data = self.check_upload({'file': 'userdata.csv'})
         eq_(data.mime.iloc[0], guess_type('userdata.csv')[0])
 
         # Large files fail
-        self.check_upload(dict(file='gramex.yaml'), code=REQUEST_ENTITY_TOO_LARGE, check=False)
+        self.check_upload({'file': 'gramex.yaml'}, code=REQUEST_ENTITY_TOO_LARGE, check=False)
         # .yaml disallowed because of allow
-        self.check_upload(dict(file='gramextest.yaml'), code=UNSUPPORTED_MEDIA_TYPE, check=False)
+        self.check_upload({'file': 'gramextest.yaml'}, code=UNSUPPORTED_MEDIA_TYPE, check=False)
         # .py disallowed because of exclude (though allow allows it)
-        self.check_upload(dict(file='server.py'), code=UNSUPPORTED_MEDIA_TYPE, check=False)
+        self.check_upload({'file': 'server.py'}, code=UNSUPPORTED_MEDIA_TYPE, check=False)
 
         # Multi-uploads are supported, with tags
-        self.check_upload(dict(file='userdata.csv', tag='t1'),
-                          dict(file='actors.csv', tag='t2'))
-        r = requests.post(self.url, files=(
-            ('file', ('x.csv', open('userdata.csv', 'rb'))),
-            ('file', ('y.csv', open('userdata.csv', 'rb'))),
-        ), data={'tag': ['t1'], 'cat': ['c1', 'c2', 'c3'], 'rand': ['x', 'y']})
+        self.check_upload(
+            {'file': 'userdata.csv', 'tag': 't1'}, {'file': 'actors.csv', 'tag': 't2'}
+        )
+        r = requests.post(
+            self.url,
+            files=(
+                ('file', ('x.csv', open('userdata.csv', 'rb'))),
+                ('file', ('y.csv', open('userdata.csv', 'rb'))),
+            ),
+            data={'tag': ['t1'], 'cat': ['c1', 'c2', 'c3'], 'rand': ['x', 'y']},
+        )
         eq_(r.status_code, OK)
+        eq_(r.headers['Paths-Exist'], '{"x.csv": true, "y.csv": true}')
         data = gramex.data.filter(self.con, table='drive').sort_values('id').tail(2)
         # If there are insufficient tags, they become NULL
         eq_(data.tag.tolist(), ['t1', None])
@@ -121,11 +135,11 @@ class TestDriveHandler(TestGramex):
         ok_('rand' not in data.columns)
 
         # ?id=..&_download downloads the file
-        data = self.check_upload(dict(file='dir/index.html'))
+        data = self.check_upload({'file': 'dir/index.html'})
         r = requests.get(self.url, params={'_download': '', 'id': data.id.iloc[0]})
         eq_(r.headers['Content-Disposition'], 'attachment; filename="index.html"')
         # TODO: FormHandler returns Content-Type using _format, so don't check for Content-Type
-        # eq_(r.headers['Content-Type'], 'text/html')
+        # Skip eq_(r.headers['Content-Type'], 'text/html')
         # Serves file with correct length despite unicode
         eq_(int(r.headers['Content-Length']), os.stat('dir/index.html').st_size)
         # If the ID is invalid, raises a NOT FOUND
@@ -137,7 +151,7 @@ class TestDriveHandler(TestGramex):
 
         # User attributes are captured on all files
         user = {'id': 'X', 'role': 'Y'}
-        data = self.check_upload(dict(file='userdata.csv'), dict(file='actors.csv'), user=user)
+        data = self.check_upload({'file': 'userdata.csv'}, {'file': 'actors.csv'}, user=user)
         for index in range(2):
             eq_(data.user_id.iloc[index], 'X')
             eq_(data.user_role.iloc[index], 'Y')
@@ -147,6 +161,8 @@ class TestDriveHandler(TestGramex):
         indices = (0, 3, 6)
         for index in indices:
             r = requests.delete(self.url, params={'id': [data.id.iloc[index]]})
+            file = data.file.iloc[index].encode('unicode-escape').decode()
+            eq_(r.headers['Paths-Exist'], '{"%s": false}' % file)
         data2 = gramex.data.filter(self.con, table='drive')
         eq_(len(data2), len(data) - len(indices))
         for index in indices:
@@ -173,10 +189,11 @@ class TestDriveHandler(TestGramex):
             'size': 100,
             'date': 100,
             'user_id': 'A',
-            'user_role': 'B'
+            'user_role': 'B',
         }
         r = requests.put(self.url, params=params)
         eq_(r.status_code, OK)
+        eq_(r.headers['Paths-Exist'], '{"a.x": true}')
         data2 = gramex.data.filter(self.con, table='drive')
         new = data2[data2.id == data.id.iloc[0]].iloc[0]
         old = data[data.id == data.id.iloc[0]].iloc[0]
@@ -193,10 +210,14 @@ class TestDriveHandler(TestGramex):
         )
         secret = gramex.service.app.settings['cookie_secret']
         user = {'id': 'AB', 'role': 'CD'}
-        r = requests.put(self.url, params=params, files=files, headers={
-            'X-Gramex-User': create_signed_value(secret, 'user', json.dumps(user))
-        })
+        r = requests.put(
+            self.url,
+            params=params,
+            files=files,
+            headers={'X-Gramex-User': create_signed_value(secret, 'user', json.dumps(user))},
+        )
         eq_(r.status_code, OK)
+        eq_(r.headers['Paths-Exist'], '{"a.x": true}')
         data2 = gramex.data.filter(self.con, table='drive')
         new = data2[data2.id == data.id.iloc[1]].iloc[0]
         old = data[data.id == data.id.iloc[1]].iloc[0]
@@ -214,6 +235,7 @@ class TestDriveHandler(TestGramex):
         params['id'] = -1
         data = gramex.data.filter(self.con, table='drive')
         r = requests.put(self.url, params=params, files=files)
+        eq_(r.headers['Paths-Exist'], '{}')
         data2 = gramex.data.filter(self.con, table='drive')
         afe(data, data2)
 
